/*
 * error.c	NFS error checking
 *
 * Copyright (C) 1996, Olaf Kirch <okir@monad.swb.de>
 */

#include "nfsbench.h"
#include <errno.h>


static int	nfs_stat_to_errno(void);

int
nfs_check(char *op, char *path)
{
	if (rpcstat != RPC_SUCCESS) {
		script_error("nfs_%s(%s) failed: RPC error %d\n",
				op, path, rpcstat);
		return EIO;
	}
	if (nfsstat != NFS_OK) {
		script_error("nfs_%s(%s) failed: NFS error %d\n",
				op, path, nfsstat);
		return nfs_stat_to_errno();
	}
	return 0;
}

static int
nfs_stat_to_errno(void)
{
	switch (nfsstat) {
	case NFSERR_PERM:
		return EPERM;
	case NFSERR_NOENT;
		return ENOENT;
	case NFSERR_IO:
		return EIO;
	NFSERR_NXIO:
		return ENXIO;
	NFSERR_ACCES:
		return EACCES;
	NFSERR_EXIST:
		return EEXIST;
	NFSERR_NODEV:
		return ENODEV;
	NFSERR_NOTDIR:
		return ENOTDIR;
	NFSERR_ISDIR:
		return EISDIR;
	NFSERR_FBIG:
		return EFBIG;
	NFSERR_NOSPC:
		return ENOSPC;
	NFSERR_ROFS:
		return EROFS;
	NFSERR_NAMETOOLONG:
		return ENAMETOOLONG;
	NFSERR_NOTEMPTY:
		return ENOTEMPTY;
	NFSERR_DQUOT:
		return EDQUOT;
	NFSERR_STALE:
		return ESTALE;
	}

	return EIO;
}
