/*
 * inode.c	getattr/setattr for nfsbench
 *
 * Copyright (C) 1996, Olaf Kirch <okir@monad.swb.de>
 */

#include <string.h>
#include "nfsbench.h"
#include "fs/nfs.h"

static void
nfs_getattr_reply(struct rpcreq *req)
{
	nfs_attrstat	*resp;

	resp = (nfs_attrstat *) req->rq_clntdata;
	xdr_nfs_getattr_res(req, &resp->nfsstat, &resp->fattr);
	resp->rpcstat = req->rq_rpcstat;
}

void
nfs_getattr(sockaddr_in *server, nfs_fh *argp, nfs_attrstat *resp)
{
	struct rpcreq	*req;

	req = generate_request(server, nfs_getattr_reply, resp);
	xdr_nfs_getattr(req, argp);
	exec_request(req);
}

static void
nfs_setattr_reply(struct rpcreq *req)
{
	nfs_attrstat	*resp;

	resp = (nfs_attrstat *) req->rq_clntdata;
	xdr_nfs_setattr_res(req, &resp->nfsstat, &resp->fattr);
	resp->rpcstat = req->rq_rpcstat;
}

void
nfs_setattr(sockaddr_in *server, nfs_setattrargs *argp, nfs_attrstat *resp)
{
	struct rpcreq	*req;

	req = generate_request(server, nfs_setattr_reply, resp);
	xdr_nfs_setattr(req, &argp->fh, &argp->sattr);;
	exec_request(req);
}

static void
nfs_statfs_reply(struct rpcreq *req)
{
	nfs_statfsres	*resp;

	resp = (nfs_statfsres *) req->rq_clntdata;
	xdr_nfs_statfs_res(req, &resp->nfsstat, &resp->stats);
	resp->rpcstat = req->rq_rpcstat;
}

void
nfs_statfs(sockaddr_in *server, nfs_fh *argp, nfs_statfsres *resp)
{
	struct rpcreq	*req;

	req = generate_request(server, nfs_statfs_reply, resp);
	xdr_nfs_statfs(req, argp);
	exec_request(req);
}

/* This didn't fit anywhere, so why not here */
static void
nfs_null_reply(struct rpcreq *req)
{
	u32	*rstatp;

	rstatp = (u32 *) req->rq_clntdata;
	xdr_nfs_null(req);
	*rstatp = req->rq_rpcstat;
}

void
nfs_null(sockaddr_in *server, u32 payload, u32 *rstatp)
{
	struct rpcreq	*req;

	req = generate_request(server, nfs_null_reply, rstatp);
	xdr_nfs_null(req);
	req->rq_msglen += payload;
	exec_request(req);
}
