/*
 * mount.c	Mount a remote directory
 *
 * Copyright (C) 1996, Olaf Kirch <okir@monad.swb.de>
 */

#include <sys/types.h>
#include <sys/socket.h>
#include "nfsbench.h"
#include "fs/mount.h"
#include "fs/portmap.h"
#include "xdr/mount.h"

struct mount_res {
	u32		rpcstat;
	u32		mountstat;
	struct nfs_fh *	fh;
};

static unsigned short	mount_port = 0;

static void
mount_mnt_reply(struct rpcreq *req)
{
	struct mount_res	*resp;

	resp = (struct mount_res *) req->rq_clntdata;
	xdr_mount_mnt_res(req, &resp->mountstat, resp->fh);
	resp->rpcstat = req->rq_rpcstat;
}

void
mount_mnt(char *path, struct nfs_fh *fh, u32 *rstatp, u32 *mstatp)
{
	struct mount_res	res;
	struct rpcreq		*req;
	sockaddr_in		server;

	if (mount_port == 0
	 && !pmap_getport(MOUNT_PROGRAM, MOUNT_VERSION, &mount_port)) {
		*rstatp = RPC_PROGNOTREGISTERED;
		return;
	}

	server.sin_family = AF_INET;
	server.sin_addr = nfs_addr;
	server.sin_port = mount_port;
	res.fh		= fh;

	while (!(req = generate_request(&server)))
		biod_flush();

	req->rq_callback = mount_mnt_reply;
	req->rq_clntdata = &res;

	xdr_mount_mnt(req, path);

	exec_request(req);

	*rstatp = res.rpcstat;
	*mstatp = res.mountstat;
}
