/*
 * portmap.c
 *
 * Copyright (C) 1996, Olaf Kirch <okir@monad.swb.de>
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <errno.h>
#include "nfsbench.h"
#include "fs/portmap.h"
#include "xdr/portmap.h"

struct pmap_res {
	int		errno;
	struct pmap	pmap;
};

static void
pmap_getport_reply(struct rpcreq *req)
{
	struct pmap_res	*resp;

	resp = (struct pmap_res *) req->rq_clntdata;
	xdr_pmap_getport_res(req, &resp->pmap.pm_port);

	if (req->rq_rpcstat == RPC_SUCCESS) {
		resp->errno = 0;
	} else {
		resp->errno = EIO;
	}
}

int
pmap_getport(u32 prog, u32 vers, unsigned short *portp)
{
	struct pmap_res	res;
	struct rpcreq	*req;
	sockaddr_in	server;

	res.pmap.pm_prog = prog;
	res.pmap.pm_vers = vers;
	res.pmap.pm_prot = IPPROTO_UDP;
	res.pmap.pm_port = 0;

	server.sin_family = AF_INET;
	server.sin_addr = nfs_addr;
	server.sin_port = htons(PMAP_PORT);

	if (!(req = generate_request(&server))) {
		biod_flush();
		req = generate_request(&server);
	}
	req->rq_callback = pmap_getport_reply;
	req->rq_clntdata = &res;

	xdr_pmap_getport(req, &res.pmap);

	enqueue_request(req);
	biod_flush();

	if (res.errno == 0)
		*portp = ntohs((unsigned short) res.pmap.pm_port);

	return res.errno;
}
