/*
 * shell/procs.h
 *
 * Macros to provide the client RPC TCL bindings
 *
 * Copyright (C) 1996, Olaf Kirch <okir@monad.swb.de>
 */

#ifndef SHELL_PROCS_H
#define SHELL_PROCS_H

#include <nfshell.h>
#include <macros.h>
#include <shell/args.h>
#include <shell/rpc.h>

#define TCL_PROC_NAME(prot, name)	CONCAT3(prot, _tclproc_, name)
#define TCL_PROC_NUMBER(prot, proc)	CONCAT3(prot, PROC_, proc)
#define TCL_PROC_PROTO \
    (ClientData cd, Tcl_Interp *interp, int argc, char **argv)
#define TCL_PROC_INIT(argt) \
    tcl_init_type(&TCL_TYPENAME(argt), (caddr_t) &args)
#define TCL_PROC_PARSE(argt) \
    tcl_parse_args(interp, &TCL_TYPENAME(argt), (caddr_t) &args, argc, argv)
#define TCL_PROC_RETURN(rest) \
    tcl_print_type(interp, &TCL_TYPENAME(rest), (caddr_t) &res)
#define TCL_PROC_RPCERROR(prot, name, status) \
    tcl_error(interp, STRING(prot) "_" STRING(name) ": %s", rpc_strerror(status))
#define TCL_PROT_NAME(prot)		STRING(prot)
#define TCL_PROT_SWITCH(prot)		CONCAT3(tcl_, prot, _switch)
#define TCL_PROT_PRIVATE(prot)		CONCAT3(TCL_, prot, _PRIVATE)
#define TCL_PROT_REGISTER(prot)		CONCAT3(tcl_, prot, _register)

#define RPC_PROT_FUNC(proc, name, argt, rest)				\
static int								\
TCL_PROC_NAME(RPC_PROT_NAME, name)TCL_PROC_PROTO			\
{									\
	struct rpcprog		*prog = (struct rpcprog *) cd;		\
	int			error, status;				\
	argt			args;					\
	rest			res;					\
									\
	TCL_PROC_INIT(argt);						\
	if ((error = TCL_PROC_PARSE(argt)) != TCL_OK)			\
		return error;						\
	status = rpc_call(prog, TCL_PROC_NUMBER(RPC_PROT_CAPNAME, proc), &args, &res);	\
	if (status != RPC_SUCCESS)					\
		return TCL_PROC_RPCERROR(RPC_PROT_NAME, name, status);	\
	return TCL_PROC_RETURN(rest);					\
}

/*
 * This instantiates all tcl functions for the protocol. Neat, isn't it?
 */
RPC_PROTOCOL

#undef RPC_PROT_FUNC

/*
 * Implement the method switch for this protocol
 */
#define RPC_PROT_FUNC(proc, name, argt, rest)				\
	if (!strcmp(argv[1], STRING(name))) {				\
		error = TCL_PROC_NAME(RPC_PROT_NAME, name)(cd, interp, 	\
						argc, argv);		\
	} else

static int
TCL_PROT_SWITCH(RPC_PROT_NAME)TCL_PROC_PROTO
{
	int		error;

	RPC_PROTOCOL
	TCL_PROT_PRIVATE(RPC_PROT_CAPNAME)
		return tcl_client_switch(cd, interp, argc, argv);

	return error;
}
#undef RPC_PROT_FUNC

void
TCL_PROT_REGISTER(RPC_PROT_NAME)(Tcl_Interp *interp, struct rpcprog *prog)
{
	static int	number = 0;
	char		namebuf[64];

	sprintf(namebuf, TCL_PROT_NAME(RPC_PROT_NAME) "%04d", ++number);
	Tcl_CreateCommand(interp, namebuf, TCL_PROT_SWITCH(RPC_PROT_NAME),
				(ClientData) prog,
				(Tcl_CmdDeleteProc *) rpc_clntdestroy);
	Tcl_SetResult(interp, namebuf, TCL_VOLATILE);
}

#endif /* SHELL_PROCS_H */
