/*
 * nfs.h	Protocol definition for NFSv2.
 *
 * Originally generated by rpcgen, with heavy manual editing.
 */
#ifndef PROTO_NFS_H
#define PROTO_NFS_H

#include <sys/time.h>

#define NFS_PROGRAM		100003
#define NFS_VERSION		2
#define NFS_PORT		2049

#define NFS_MAXPATHLEN		1024
#define NFS_MAXNAMLEN		255
#define NFS_FHSIZE		32
#define NFS_COOKIESIZE		4
#define NFS_FIFO_DEV		-1
#define NFSMODE_FMT		0170000
#define NFSMODE_DIR		0040000
#define NFSMODE_CHR		0020000
#define NFSMODE_BLK		0060000
#define NFSMODE_REG		0100000
#define NFSMODE_LNK		0120000
#define NFSMODE_SOCK		0140000
#define NFSMODE_FIFO		0010000

#define NFSPROC_NUL		0
#define NFSPROC_GETATTR		1
#define NFSPROC_SETATTR		2
#define NFSPROC_ROOT		3
#define NFSPROC_LOOKUP		4
#define NFSPROC_READLINK	5
#define NFSPROC_READ		6
#define NFSPROC_WRITECACHE	7
#define NFSPROC_WRITE		8
#define NFSPROC_CREATE		9
#define NFSPROC_REMOVE		10
#define NFSPROC_RENAME		11
#define NFSPROC_LINK		12
#define NFSPROC_SYMLINK		13
#define NFSPROC_MKDIR		14
#define NFSPROC_RMDIR		15
#define NFSPROC_READDIR		16
#define NFSPROC_STATFS		17

enum {
	NFS_OK			= 0,
	NFSERR_PERM		= 1,
	NFSERR_NOENT		= 2,
	NFSERR_IO		= 5,
	NFSERR_NXIO		= 6,
	NFSERR_ACCES		= 13,
	NFSERR_EXIST		= 17,
	NFSERR_NODEV		= 19,
	NFSERR_NOTDIR		= 20,
	NFSERR_ISDIR		= 21,
	NFSERR_FBIG		= 27,
	NFSERR_NOSPC		= 28,
	NFSERR_ROFS		= 30,
	NFSERR_NAMETOOLONG	= 63,
	NFSERR_NOTEMPTY		= 66,
	NFSERR_DQUOT		= 69,
	NFSERR_STALE		= 70,
	NFSERR_WFLUSH		= 99,
};

enum {
	NFNON			= 0,
	NFREG			= 1,
	NFDIR			= 2,
	NFBLK			= 3,
	NFCHR			= 4,
	NFLNK			= 5,
	NFSOCK			= 6,
	NFBAD			= 7,
	NFFIFO			= 8,
};

typedef int			nfs_null;
typedef int			nfs_nullres;
typedef u32			nfs_stat;

typedef struct nfs_fh {
	char data[32];
} nfs_fh;

typedef struct nfs_fattr {
	u32		type;
	u32		mode;
	u32		nlink;
	u32		uid;
	u32		gid;
	u32		size;
	u32		blocksize;
	u32		rdev;
	u32		blocks;
	u32		fsid;
	u32		fileid;
	struct timeval	atime;
	struct timeval	mtime;
	struct timeval	ctime;
} nfs_fattr;

typedef struct nfs_sattr {
	u32		mode;
	u32		uid;
	u32		gid;
	u32		size;
	struct timeval	atime;
	struct timeval	mtime;
} nfs_sattr;

typedef struct nfs_attrstat {
	u32		rpcstat;
	nfs_stat	nfsstat;
	nfs_fattr	fattr;
} nfs_attrstat;

typedef struct nfs_sattrargs {
	nfs_fh		fh;
	nfs_sattr	sattr;
} nfs_sattrargs;

typedef struct nfs_diropargs {
	nfs_fh		fh;
	char *		name;
} nfs_diropargs;

typedef struct nfs_diropres {
	u32		rpcstat;
	nfs_stat	nfsstat;
	nfs_fh		fh;
	nfs_fattr	fattr;
} nfs_diropres;

typedef struct nfs_readlinkres {
	u32		rpcstat;
	nfs_stat	nfsstat;
	char *		path;
} nfs_readlinkres;

typedef struct nfs_readargs {
	nfs_fh		fh;
	u32		offset;
	u32		count;
} nfs_readargs;

typedef struct nfs_data {
	u32		count;
	u32 *		data;
} nfs_data;

typedef struct nfs_readres {
	u32		rpcstat;
	nfs_stat	nfsstat;
	nfs_fattr	fattr;
	/* u32		count; */
	nfs_data	data;
} nfs_readres;

typedef struct nfs_writeargs {
	nfs_fh		fh;
	u32		offset;
	/* u32		count; */
	nfs_data	data;
} nfs_writeargs;

typedef struct nfs_createargs {
	nfs_fh		fh;
	char *		name;
	nfs_sattr	sattr;
} nfs_createargs;

typedef struct nfs_renameargs {
	nfs_fh		ffh;
	char *		fname;
	nfs_fh		tfh;
	char *		tname;
} nfs_renameargs;

typedef struct nfs_linkargs {
	nfs_fh		ffh;
	nfs_fh		tfh;
	char *		tname;
} nfs_linkargs;

typedef struct nfs_symlinkargs {
	nfs_fh		ffh;
	char *		fname;
	char *		path;
	nfs_sattr	sattr;
} nfs_symlinkargs;

typedef struct nfs_readdirargs {
	nfs_fh		fh;
	u32		cookie;
	u32		count;
} nfs_readdirargs;

typedef void		(*nfs_dirent_fn_t)(void *, char *, u32);

typedef struct nfs_callback {
	nfs_dirent_fn_t	func;
	void *		data;
} nfs_callback;

typedef struct nfs_readdirres {
	u32		rpcstat;
	nfs_stat	nfsstat;
	nfs_callback	iter;
} nfs_readdirres;

typedef struct nfs_fstat {
	u32		tsize;
	u32		bsize;
	u32		blocks;
	u32		bfree;
	u32		bavail;
} nfs_fstat;

typedef struct nfs_statfsres {
	u32		rpcstat;
	nfs_stat	nfsstat;
	nfs_fstat	stats;
} nfs_statfsres;

extern int	nfsstat;

#if 0
unsigned int xdr_nfs_null(struct rpcreq *, void *);
unsigned int xdr_nfs_nullres(struct rpcreq *, void *);
unsigned int xdr_nfs_getattr(struct rpcreq *, nfs_fh *);
unsigned int xdr_nfs_attrstat(struct rpcreq *, nfs_attrstat *);
unsigned int xdr_nfs_setattr(struct rpcreq *, nfs_setattrargs *);
unsigned int xdr_nfs_root(struct rpcreq *, void *);
unsigned int xdr_nfs_stat(struct rpcreq *, u32 *);
unsigned int xdr_nfs_lookup(struct rpcreq *, nfs_diropargs *);
unsigned int xdr_nfs_diropres(struct rpcreq *, nfs_diropres *);
unsigned int xdr_nfs_readlink(struct rpcreq *, nfs_fh *);
unsigned int xdr_nfs_readlinkres(struct rpcreq *, nfs_readlinkres *);
unsigned int xdr_nfs_read(struct rpcreq *, nfs_readargs *);
unsigned int xdr_nfs_readres(struct rpcreq *, nfs_readres *);
unsigned int xdr_nfs_writecache(struct rpcreq *, void *);
unsigned int xdr_nfs_writecacheres(struct rpcreq *, u32 *);
unsigned int xdr_nfs_write(struct rpcreq *, nfs_writeargs *);
unsigned int xdr_nfs_create(struct rpcreq *, nfs_createargs *);
unsigned int xdr_nfs_rename(struct rpcreq *, nfs_renameargs *);
unsigned int xdr_nfs_link(struct rpcreq *, nfs_linkargs *);
unsigned int xdr_nfs_symlink(struct rpcreq *, nfs_symlinkargs *);
unsigned int xdr_nfs_readdir(struct rpcreq *, nfs_readdirargs *);
unsigned int xdr_nfs_readdirres(struct rpcreq *, nfs_readdirres *);
unsigned int xdr_nfs_statfs(struct rpcreq *, nfs_fh *);
unsigned int xdr_nfs_statfsres(struct rpcreq *, nfs_statfsres *);
#endif

extern rqxdrproc_t		nfs_encode[];
extern rqxdrproc_t		nfs_decode[];

char *	xdr_nfs_name(u32 op);

#endif /* PROTO_NFS_H */
