/*
 * include/xdr/nlm.h
 *
 * Declarations for the NLM protocol
 *
 * Copyright (C) 1996, Olaf Kirch <okir@monad.swb.de>
 */

#ifndef PROTO_NLM_H
#define PROTO_NLM_H

#include <xdr/nfs.h>

#define NLM_PROGRAM		100021
#define NLM_VERSION		2
#define NLMPROC_NUL		0
#define NLMPROC_TEST		1
#define NLMPROC_LOCK		2
#define NLMPROC_CANCEL		3
#define NLMPROC_UNLOCK		4
#define NLMPROC_GRANTED		5
#define NLMPROC_TEST_MSG	6
#define NLMPROC_LOCK_MSG	7
#define NLMPROC_CANCEL_MSG	8
#define NLMPROC_UNLOCK_MSG	9
#define NLMPROC_GRANTED_MSG	10
#define NLMPROC_TEST_RES	11
#define NLMPROC_LOCK_RES	12
#define NLMPROC_CANCEL_RES	13
#define NLMPROC_UNLOCK_RES	14
#define NLMPROC_GRANTED_RES	15
#define NLMPROC_SHARE		20
#define NLMPROC_UNSHARE		21
#define NLMPROC_NM_LOCK		22
#define NLMPROC_FREE_ALL	23

#define nlm_null		int
#define nlm_nullres		int

enum {
	NLM_LCK_GRANTED	= 0,
	NLM_LCK_DENIED,
	NLM_LCK_DENIED_NOLOCKS,
	NLM_LCK_BLOCKED,
	NLM_LCK_DENIED_GRACE_PERIOD,
};

typedef struct nlm_netobj {
	u32			len;
	unsigned char *		data;
} nlm_netobj;

typedef char *			nlm_owner;

typedef struct nlm_lock {
	char *			caller;
	nfs_fh			fhandle;
	nlm_owner		owner;
	u32			pid;
	u32			offset;
	u32			length;
} nlm_lock;

typedef struct nlm_holder {
	u32			excl;
	u32			pid;
	nlm_owner		owner;
	u32			offset;
	u32			length;
} nlm_holder;

typedef struct nlm_testargs {
	u32			cookie;
	u32			excl;
	nlm_lock		lock;
} nlm_testargs;

typedef struct nlm_lockargs {
	u32			cookie;
	u32			block;
	u32			excl;
	nlm_lock		lock;
	u32			reclaim;
	u32			state;
} nlm_lockargs;

typedef struct nlm_cancargs {
	u32			cookie;
	u32			block;
	u32			excl;
	nlm_lock		lock;
} nlm_cancargs;

typedef struct nlm_unlockargs {
	u32			cookie;
	nlm_lock		lock;
} nlm_unlockargs;

typedef struct nlm_res {
	u32			cookie;
	u32			status;
} nlm_res;

typedef struct nlm_testres {
	u32			cookie;
	u32			status;
	nlm_holder		holder;
} nlm_testres;

extern rqxdrproc_t		nlm_encode[];
extern rqxdrproc_t		nlm_decode[];

#endif /* PROTO_NLM_H */
