/*
 * rpc.h	General RPC declarations.
 *
 * Copyright (C) 1996, Olaf Kirch <okir@monad.swb.de>
 */

#ifndef PROTO_RPC_H
#define PROTO_RPC_H

enum {
	RPC_SUCCESS=0,			/* call succeeded */
	RPC_CANTENCODEARGS=1,		/* can't encode arguments */
	RPC_CANTDECODERES=2,		/* can't decode results */
	RPC_CANTSEND=3,			/* failure in sending call */
	RPC_CANTRECV=4,			/* failure in receiving result */
	RPC_TIMEDOUT=5,			/* call timed out */
	RPC_VERSMISMATCH=6,		/* rpc versions not compatible */
	RPC_AUTHERROR=7,		/* authentication error */
	RPC_PROGUNAVAIL=8,		/* program not available */
	RPC_PROGVERSMISMATCH=9,		/* program version mismatched */
	RPC_PROCUNAVAIL=10,		/* procedure unavailable */
	RPC_CANTDECODEARGS=11,		/* decode arguments error */
	RPC_SYSTEMERROR=12,		/* generic "other problem" */
	RPC_UNKNOWNHOST=13,		/* unknown host name */
	RPC_UNKNOWNPROTO=17,		/* unkown protocol */
	RPC_PMAPFAILURE=14,		/* the pmapper failed in its call */
	RPC_PROGNOTREGISTERED=15,	/* remote program is not registered */
	RPC_FAILED=16
};

extern int	auth_uid;
extern int	auth_gid;

u32 *		rpc_encode_header(u32 *msg, u32 prog, u32 vers, u32 proc);
unsigned int	rpc_decode_header(struct svcreq *, u32 **);
u32 *		rpc_encode_reply(struct svcreq *, u32 *, u32);
unsigned int	rpc_decode_reply(struct rpcreq *, u32 **);
const char *	rpc_strerror(u32 stat);

/*
 * Request length adjustment
 */
extern inline unsigned int
xdr_adjust_msglen(struct rpcreq *req, u32 *msg)
{
	return (req->rq_msglen = (msg - req->rq_msg) << 2);
}

/*
 * Request length check 
 */
extern inline unsigned int
xdr_check_msglen(struct svcreq *req, u32 *msg)
{
	unsigned int	len;

	len = (msg - req->rq_msg) << 2;
	if (len <= req->rq_msglen)
		return len;

	/* complain about short packet */
	req->rq_rpcstat = 4;		/* GARBAGE_ARGS */
	return 0;
}

/*
 * Reply length adjustment
 */
extern inline unsigned int
xdr_adjust_rplylen(struct svcreq *req, u32 *msg)
{
	return (req->rq_rplylen = (msg - req->rq_rply) << 2);
}

/*
 * Reply length check
 */
extern inline unsigned int
xdr_check_rplylen(struct rpcreq *req, u32 *msg)
{
	unsigned int	len;

	len = (msg - req->rq_rply) << 2;
	if (len <= req->rq_rplylen)
		return len;

	/* complain about short packet */
	req->rq_rpcstat = RPC_CANTDECODERES;
	return 0;
}

#endif /* PROTO_RPC_H */
