/*
 * xdr/xdrdefs.h
 *
 * Template for XDR definitions of an RPC protocol
 *
 * Copyright (C) 1996, Olaf Kirch <okir@monad.swb.de>
 */

#ifndef XDRDEFS_H
#define XDRDEFS_H

#define XDR_TABLE_NAME(prot, tag)	CONCAT2(prot, tag)
#define XDR_TAGGED_NAME(prot, tag)	CONCAT3(xdr_, prot, tag)

/*
 * Mapping of procedure numbers to XDR encode functions
 */
#define RPC_PROT_FUNC(proc, name, argt, rest)	(rqxdrproc_t) xdr_##argt,
rqxdrproc_t			XDR_TABLE_NAME(RPC_PROT_NAME, _encode)[] = {
	RPC_PROTOCOL
};
#undef RPC_PROT_FUNC

/*
 * Mapping of procedure numbers to XDR decode functions
 */
#define RPC_PROT_FUNC(proc, name, argt, rest)	(rqxdrproc_t) xdr_##rest,
rqxdrproc_t			XDR_TABLE_NAME(RPC_PROT_NAME, _decode)[] = {
	RPC_PROTOCOL
};
#undef RPC_PROT_FUNC



/*
 * Server part of the protocol
 */
#ifdef RPC_SERVER_PROTO
/* Mapping of procedure numbers to XDR request decode functions */
#define RPC_PROT_FUNC(proc, name, argt, rest)	(svxdrproc_t) xdrsvc_##argt,
svxdrproc_t			XDR_TABLE_NAME(RPC_PROT_NAME, _svcdecode)[] = {
	RPC_PROTOCOL
};
#undef RPC_PROT_FUNC

/* Mapping of procedure numbers to XDR reply encode functions */
#define RPC_PROT_FUNC(proc, name, argt, rest)	(svxdrproc_t) xdrsvc_##rest,
svxdrproc_t			XDR_TABLE_NAME(RPC_PROT_NAME, _svcencode)[] = {
	RPC_PROTOCOL
};
#undef RPC_PROT_FUNC
#endif

/*
 * Mapping of proc numbers to names
 */
#define RPC_PROT_FUNC(proc, name, argt, rest)	# name
#define RPC_PROC_NAMES		XDR_TABLE_NAME(RPC_PROT_NAME, _names)
static char *			RPC_PROC_NAMES[] = {
	RPC_PROTOCOL
};
#undef RPC_PROTOCOL

char *
XDR_TAGGED_NAME(RPC_PROT_NAME, _name)(u32 op)
{
	if (op < (sizeof(RPC_PROC_NAMES)/sizeof(RPC_PROC_NAMES[0])))
		return RPC_PROC_NAMES[op];
	return "unknown";
}
#undef RPC_PROC_NAMES
#undef RPC_PROT_FUNC

#endif /* XDRDEFS_H */
