/*
 * rpc/request.c
 *
 * Request creation and scheduling
 *
 * Copyright (C) 1996, Olaf Kirch <okir@monad.swb.de>
 */

#include <sys/types.h>
#include <sys/param.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <errno.h>

#include "nfshell.h"

struct rpcreq *		req_free = NULL;
struct rpcreq *		req_pending = NULL;
struct rpcreq *		req_pending_tail = NULL;
struct rpcreq *		req_queued = NULL;
struct rpcreq *		req_queued_tail = NULL;

struct rpcreq *
generate_request(sockaddr_in *addr, rqcallback_t func, void *data)
{
	static int	inited = 0;
	struct rpcreq	*req;

	if (!inited) {
		u32	*msg;
		int	i;

		for (i = 0; i < MAXREQUESTS; i++) {
			req = (struct rpcreq *) malloc(sizeof(*req));
			msg = (u32 *) malloc(4 * MAXPKTSIZE);
			if (!req || !msg) {
				fprintf(stderr, "malloc failed, abort!\n");
				exit(2);
			}
			memset(req, 0, sizeof(*req));
			req->rq_msg = msg;
			req->rq_next = req_free;
			req_free = req;
		}
		inited = 1;
	}

	while (!(req = req_free))
		biod_flush();

	req_free = req->rq_next;
	req->rq_callback = func;
	req->rq_clntdata = data;
	req->rq_addr     = addr;

	return req;
}

void
insert_request(struct rpcreq **que, struct rpcreq *req)
{
	for (; *que; que = &((*que)->rq_next)) {
		if (req->rq_expires < (*que)->rq_expires)
			break;
	}
	req->rq_next = *que;
	*que = req;
}

void
enqueue_request(struct rpcreq *req)
{
	req->rq_xid = req->rq_msg[0];
	req->rq_rexmits = 0;
	req->rq_timeout = INITIMEO;
	req->rq_expires = ~0;
	req->rq_qtime = 0;

	append_request(req_queued, req_queued_tail, req);
	/*
	printf("enqueue_request = %08lx ptr %p nxt %p\n",
			(unsigned long) req->rq_xid,
			req, req->rq_next);
	 */
}

void
finish_request(struct rpcreq *req, u32 *reply, unsigned int len)
{
	if (req->rq_callback) {
		req->rq_rply    = reply;
		req->rq_rplylen = len;
		req->rq_callback(req);
	}

	req->rq_next = req_free;
	req_free = req;
}
