/*
 * shell/nlm_args.c
 *
 * Define the argument conversion for the NLMv2 binding
 *
 * Copyright (C) 1996, Olaf Kirch <okir@monad.swb.de>
 */

#include <memory.h>
#include <malloc.h>
#include <nfshell.h>
#include <xdr/nlm.h>
#include <shell/nfs.h>
#include <shell/nlm.h>

TCL_DECLARE_TYPE(extern, u32);

/*
 * TCL handling for NLM status
 */
#define tcl_nlm_stat_init	NULL

static int
tcl_nlm_stat_parse(Tcl_Interp *interp, void *d, u32 *s, char *a)
{
	if (!strcasecmp(a, "lck_granted"))
		*s = NLM_LCK_GRANTED;
	else
	if (!strcasecmp(a, "lck_denied"))
		*s = NLM_LCK_DENIED;
	else
	if (!strcasecmp(a, "lck_denied_nolocks"))
		*s = NLM_LCK_DENIED_NOLOCKS;
	else
	if (!strcasecmp(a, "lck_blocked"))
		*s = NLM_LCK_BLOCKED;
	else
	if (!strcasecmp(a, "lck_denied_grace_period"))
		*s = NLM_LCK_DENIED_GRACE_PERIOD;
	else
		return tcl_error(interp, "NLM: unknown status %s", a);
	return TCL_OK;
}

static int
tcl_nlm_stat_print(Tcl_Interp *interp, void *dummy, u32 *statp)
{
	char	buffer[20], *errname;

	switch (*statp) {
	case NLM_LCK_GRANTED:
		errname = "LCK_GRANTED"; break;
	case NLM_LCK_DENIED:
		errname = "LCK_DENIED"; break;
	case NLM_LCK_DENIED_NOLOCKS:
		errname = "LCK_DENIED_NOLOCKS"; break;
	case NLM_LCK_BLOCKED:
		errname = "LCK_BLOCKED"; break;
	case NLM_LCK_DENIED_GRACE_PERIOD:
		errname = "LCK_DENIED_GRACE_PERIOD"; break;
	default:
		sprintf(buffer, "nlmstat%d", (int) *statp);
		errname = buffer;
		break;
	}

	Tcl_AppendElement(interp, errname);
	/* return (*statp == NLM_LCK_DENIED)? TCL_BREAK : TCL_ERROR; */
	return TCL_OK;
}

TCL_DEFINE_TYPE(extern, nlm_stat, NULL);

/*
 * argument initialization
 */
static void
tcl_nlm_lock_init(void *dummy, nlm_lock *lock)
{
	lock->caller = nlm_hostname;
	/* lock->owner  = nlm_owner; */
	lock->pid    = nlm_pid;
}

static void
tcl_nlm_testargs_init(void *dummy, nlm_testargs *argp)
{
	memset(argp, 0, sizeof(*argp));
	argp->cookie = ++nlm_cookie;
	tcl_nlm_lock_init(dummy, &argp->lock);
}

static void
tcl_nlm_lockargs_init(void *dummy, nlm_lockargs *argp)
{
	memset(argp, 0, sizeof(*argp));
	argp->cookie = ++nlm_cookie;
	argp->state  = nlm_state;
	tcl_nlm_lock_init(dummy, &argp->lock);
}

static void
tcl_nlm_cancargs_init(void *dummy, nlm_cancargs *argp)
{
	memset(argp, 0, sizeof(*argp));
	argp->cookie = ++nlm_cookie;
	tcl_nlm_lock_init(dummy, &argp->lock);
}

static void
tcl_nlm_unlockargs_init(void *dummy, nlm_unlockargs *argp)
{
	memset(argp, 0, sizeof(*argp));
	argp->cookie = ++nlm_cookie;
	tcl_nlm_lock_init(dummy, &argp->lock);
}

/*
 * Result initialization
 */
#define tcl_nlm_res_init			NULL
#define tcl_nlm_testres_init			NULL
#define tcl_nlm_holder_init			tcl_struct_init


TCL_BEGIN_STRUCT(static, nlm_lock)
	TCL_MEMBER(nlm_lock, nfs_fh,		fhandle),
	TCL_MEMBER(nlm_holder, string,		owner),
	TCL_MEMBER(nlm_lock, u32,		offset),
	TCL_MEMBER(nlm_lock, u32,		length),
TCL_END_STRUCT(static, nlm_lock);
TCL_DEFINE_STRUCT(static, nlm_lock);

TCL_BEGIN_STRUCT(static, nlm_holder)
	TCL_MEMBER(nlm_holder, u32,		excl),
	TCL_MEMBER(nlm_holder, u32,		pid),
	TCL_MEMBER(nlm_holder, string,		owner),
	TCL_MEMBER(nlm_holder, u32,		offset),
	TCL_MEMBER(nlm_holder, u32,		length),
TCL_END_STRUCT(static, nlm_holder)
TCL_DEFINE_STRUCT(static, nlm_holder);

TCL_BEGIN_STRUCT(static, nlm_testargs)
	TCL_MEMBER(nlm_testargs, u32,		excl),
	TCL_MEMBER(nlm_testargs, nlm_lock,	lock)
TCL_END_STRUCT(static, nlm_testargs);
TCL_DEFINE_STRUCT(extern, nlm_testargs);

TCL_BEGIN_STRUCT(static, nlm_lockargs)
	TCL_MEMBER(nlm_lockargs, u32,		block),
	TCL_MEMBER(nlm_lockargs, u32,		excl),
	TCL_MEMBER(nlm_lockargs, nlm_lock,	lock),
	TCL_MEMBER(nlm_lockargs, u32,		reclaim),
TCL_END_STRUCT(static, nlm_lockargs);
TCL_DEFINE_STRUCT(extern, nlm_lockargs);

TCL_BEGIN_STRUCT(static, nlm_cancargs)
	TCL_MEMBER(nlm_cancargs, u32,		block),
	TCL_MEMBER(nlm_cancargs, u32,		excl),
	TCL_MEMBER(nlm_cancargs, nlm_lock,	lock)
TCL_END_STRUCT(static, nlm_cancargs);
TCL_DEFINE_STRUCT(extern, nlm_cancargs);

TCL_BEGIN_STRUCT(static, nlm_unlockargs)
	TCL_MEMBER(nlm_unlockargs, nlm_lock,	lock),
TCL_END_STRUCT(static, nlm_unlockargs);
TCL_DEFINE_STRUCT(extern, nlm_unlockargs);

TCL_BEGIN_STRUCT(static, nlm_res)
	TCL_MEMBER(nlm_res, nlm_stat,		status),
TCL_END_STRUCT(static, nlm_res);
TCL_DEFINE_STRUCT(extern, nlm_res);

TCL_BEGIN_STRUCT(static, nlm_testres)
	TCL_MEMBER(nlm_testres, nlm_stat,	status),
	TCL_MEMBER(nlm_testres, nlm_holder,	holder),
TCL_END_STRUCT(static, nlm_testres);
TCL_DEFINE_STRUCT(extern, nlm_testres);
