/*
 * shell/nlm_proc.c
 *
 * TCL binding for NLM
 *
 * Copyright (C) 1996, Olaf Kirch <okir@monad.swb.de>
 */

#include <nfshell.h>
#include <xdr/nlm.h>
#include <xdr/nlm_prot.h>
#include <shell/nfs.h>
#include <shell/nlm.h>

u32				nlm_cookie = 1;
u32				nlm_pid = 666;
u32				nlm_state = 25;
char				nlm_hostname[257] = "gumbo";

#define nlm_void		int

#define RPC_PROTOCOL		NLM_PROTOCOL
#define RPC_PROT_NAME		nlm
#define RPC_PROT_CAPNAME	NLM
#define TCL_NLM_PRIVATE		\
  if ((error = tcl_nlm_private(cd, interp, argc, argv)) != TCL_CONTINUE) { \
		/* NOP */;						\
  } else

static int
tcl_nlm_private(ClientData cd, Tcl_Interp *interp, int argc, char **argv)
{
	if (!strcmp(argv[1], "set_pid") || !strcmp(argv[1], "set_state")) {
		int	value;

		if (argc != 3)
			return tcl_error(interp,
					"%s usage: %s %s <int>",
					argv[0], argv[0], argv[1]);
		if (Tcl_GetInt(interp, argv[2], &value) == TCL_OK) {
			if (argv[1][4] == 'p')
				nlm_pid = (u32) value;
			else
				nlm_state = (u32) value;
			return TCL_OK;
		}
		return tcl_error(interp,
				"%s %s: bad integer value", argv[0], argv[1]);
	}

	if (!strcmp(argv[1], "set_hostname")) {
		if (argc != 3)
			return tcl_error(interp,
					"%s usage: %s %s <string>",
					argv[0], argv[0], argv[1]);
		if (strlen(argv[2]) >= sizeof(nlm_hostname))
			return tcl_error(interp,
					"%s %s: hostname too long",
					argv[0], argv[1]);
		strcpy(nlm_hostname, argv[2]);
		return TCL_OK;
	}

	return TCL_CONTINUE;
}

#include <shell/procs.h>
