/*
 * shell/types.c
 *
 * TCL argument handling for simple types.
 *
 * Copyright (C) 1996 Olaf Kirch <okir@monad.swb.de>
 */

#include <stdlib.h>
#include <shell/args.h>

/*
 * Handling of void
 */
#define tcl_void_init		NULL

TCL_BEGIN_STRUCT(static, void)
TCL_END_STRUCT(static, void)
TCL_DEFINE_STRUCT(extern, void);
#if 0
static int
tcl_void_parse(Tcl_Interp *interp, void *dummy, void *p, char *a)
{
	return tcl_error(interp, "%s: attempt to parse void argument", proc);
}

static int
tcl_void_print(Tcl_Interp *interp, void *dummy, void *p)
{
	return tcl_error(interp, "attempt to print void argument");
}

TCL_DEFINE_TYPE(extern, void, NULL);
#endif

/*
 * Handling of u32 arguments
 */
#define tcl_u32_init		NULL

static int
tcl_u32_parse(Tcl_Interp *interp, void *dummy, u32 *ptr, char *arg)
{
	int	value, error;

	error = Tcl_GetInt(interp, arg, &value);
	*ptr = (u32) value;
	return error;
}

static int
tcl_u32_print(Tcl_Interp *interp, void *dummy, u32 *ptr)
{
	char	buffer[20];

	sprintf(buffer, "%d", (int) *ptr);
	Tcl_AppendElement(interp, buffer);
	return TCL_OK;
}

TCL_DEFINE_TYPE(extern, u32, NULL);

/*
 * Handling of int arguments
 */
#define tcl_int_init		NULL

static int
tcl_int_parse(Tcl_Interp *interp, void *dummy, int *ptr, char *arg)
{
	return Tcl_GetInt(interp, arg, ptr);
}

static int
tcl_int_print(Tcl_Interp *interp, void *dummy, int *ptr)
{
	char	buffer[20];

	sprintf(buffer, "%d", *ptr);
	Tcl_AppendElement(interp, buffer);
	return TCL_OK;
}

TCL_DEFINE_TYPE(extern, int, NULL);

/*
 * Handling of struct timeval arguments
 */
static void
tcl_timeval_init(void *dummy, struct timeval *ptr)
{
	ptr->tv_sec = ptr->tv_usec = 0;
}

static int
tcl_timeval_parse(Tcl_Interp *interp, void *dummy, struct timeval *tv, char *arg)
{
	int	argc, error;
	char	**argv;

	if ((error = Tcl_SplitList(interp, arg, &argc, &argv)) != TCL_OK)
		return tcl_error(interp, "bad timeval argument");

	if (argc == 0 || argc > 2)
		return tcl_error(interp, "bad timeval argument");

	if ((error = Tcl_GetInt(interp, argv[0], &tv->tv_sec)) != TCL_OK
	 || (argc == 2
	  && (error = Tcl_GetInt(interp, argv[1], &tv->tv_usec)) != TCL_OK))
		return tcl_error(interp, "timeval arguments must be integer");

	free(argv);
	return TCL_OK;
}

static int
tcl_timeval_print(Tcl_Interp *interp, void *dummy, struct timeval *tv)
{
	char	buffer[40];

	sprintf(buffer, "%d %d", tv->tv_sec, tv->tv_usec);
	Tcl_AppendElement(interp, buffer);
	return TCL_OK;
}

TCL_DEFINE_TYPE(extern, timeval, NULL);

/*
 * Handling of string arguments
 */
static void
tcl_string_init(void *dummy, char **name)
{
	*name = "unknown";
}

static int
tcl_string_parse(Tcl_Interp *interp, void *d, char **name, char *arg)
{
	*name = arg;
	return TCL_OK;
}

static int
tcl_string_print(Tcl_Interp *interp, void *d, char **name)
{
	Tcl_AppendElement(interp, *name);
	return TCL_OK;
}

TCL_DEFINE_TYPE(extern, string, NULL);
