/*
 * mount.c	XDR support for the NFS mount protocol version 1
 *		Only MNT/UMNT/UMNTALL supported; the rest would require
 *		TCP and I don't feel like that today.
 *
 * Copyright (C) 1996, Olaf Kirch <okir@monad.swb.de>
 */

#include <sys/types.h>
#include <string.h>
#include "nfshell.h"
#include "macros.h"
#include "xdr/nfs.h"
#include "xdr/mount.h"
#include "xdr/mount_prot.h"

#define ENC	1
#define DEC	0

/*
 * XDR encode/decode functions.
 */
static u32 *
xdr_nfs_fh(u32 *msg, int enc, nfs_fh *fh)
{
	if (enc)
		memcpy(msg, fh, sizeof(*fh));
	else
		memcpy(fh, msg, sizeof(*fh));
	msg += (sizeof(*fh) + 3) >> 2;
	return msg;
}

static u32 *
xdr_mnt_fname(u32 *msg, int enc, char *fname)
{
	int	len;

	if (enc) {
		len = strlen(fname);
		*msg++ = htonl(len);
		strcpy((char *) msg, fname);
	} else {
		len = ntohl(*msg++);
		memcpy(fname, msg, len);
		fname[len] = 0;
	}
	msg += ((len + 3) >> 2);
	return msg;
}

/*
static inline unsigned int
xdr_mnt_reply(struct rpcreq *req, u32 **msgp)
{
	return rpc_reply(req, msgp);
}

static inline unsigned int
xdr_mnt_void_res(struct rpcreq *req)
{
	u32		*msg = req->rq_rply;

	if (xdr_mnt_reply(req, &msg) != 0)
		return xdr_check_rplylen(req, msg);
	return 0;
}
 */

/*
 * MOUNT async procedures.
 */
static u32 *
xdr_mnt_null(struct rpcreq *req, u32 *msg, void *dummy)
{
	return msg;
}

static u32 *
xdr_mnt_nullres(struct rpcreq *req, u32 *msg, void *dummy)
{
	return msg;
}

static u32 *
xdr_mnt_dirpath(struct rpcreq *req, u32 *msg, mnt_dirpath *argp)
{
	return xdr_mnt_fname(msg, ENC, *argp);
}

static u32 *
xdr_mnt_fhstatus(struct rpcreq *req, u32 *msg, mnt_fhstatus *resp)
{
	if ((resp->status = ntohl(*msg++)) == 0) {
		msg = xdr_nfs_fh(msg, DEC, &resp->fh);
	}
	return msg;
}

static u32 *
xdr_mnt_mountlist(struct rpcreq *req, u32 *msg, mnt_mountlist *resp)
{
	return NULL;
}

static u32 *
xdr_mnt_exportlist(struct rpcreq *req, u32 *msg, mnt_exportlist *resp)
{
	return NULL;
}

#define RPC_PROTOCOL		MOUNT_PROTOCOL
#define RPC_PROT_NAME		mnt
#include <xdr/xdrdefs.h>
